!     ------------------------------------------------------------------
!
!     BJT.f95
!
!
!     Created on Thu Apr 12 20:32:51 2007
!     Copyright (c) 2007 James K Beard. All rights reserved.
!
!
!     ------------------------------------------------------------------

module BJT	!Engine for simplified v-i curve family for BJT
implicit none
!Elementary model
!Saturation equation is IC = V_CE/Saturation_resistance
!Constant current equation is IC = ISat+beta*IB+V_CE/Collector_Resistance
!The smaller of these is the output
!The intersection of these lines is always included in the output table

!Global transistor parameters, with defaults
real(kind(0.d0))::		&
  beta=350,			&
  Saturation_Resistance=45d0,	&	!45 Ohms
  V_CE_Early=100d0,		&	!100 Volts
!  Collector_Resistance=1d5,	&	!100 kOhms
  IC0=0.0000002			!200 pA collector leakange current
!Global plotting parameters, with defaults
real(kind(0.d0))::		&
  IBdelta=10.d-6 			!10 uA
integer::NIbase=6			!0 to 60 uA
real(kind(0d0))::		&
  V_CEdelta=1.d0,		&	!1 Volt increments
  V_CEmax=20d0				!0 to 20 Volts
real(kind(0.d0))::IBase,VCollector,VCIntercept
integer::jIB

!Public entry points
!subroutine BJT_vi_Curves(lunout)
!real(kind(0.d0)) function BJT_IC_vs_VC(V_CE,IB)
!real(kind(0.d0)) function BJT_Output_Admittance(VC,IB)

!------------------------------
contains
!------------------------------

subroutine BJT_vi_Curves(lunout)
integer,intent(IN)::lunout
  open(lunout,file="BJT_Transistor_vi.csv")
  call title_line()

  call model_line(0.d0)	!Output first line of file

  do jIB=0,NIbase	!Write the intersections
    IBase=jIB*IBDelta	!For each base current
    VCIntercept=(IC0+beta*IBase)*Saturation_Resistance	!Intersection of the lines
    call model_line(VCIntercept)
  end do

  VCollector=0.d0
  do
    VCollector=VCollector+V_CEdelta
    if(VCollector < VCIntercept) cycle	!Skip if not past the intersections
    if(VCollector > V_CEmax) exit	!Done when past the limit
    call model_line(VCollector)
  end do

  close(lunout)

!-------------------------
contains
!----------------------------------

subroutine title_line()
integer::i
real(kind(0.d0)),dimension(NIBase+1)::IBTitle

  IBTitle=(/(i,i=0,NIBase)/)*IBDelta
  write(lunout,1000) IBTitle*1.d6	!Output in uA

1000 format("V_CE V"20(:",",f4.1," uA"))
end subroutine title_line
!------------------------------------
subroutine model_line(VC)
real(kind(0.d0)),intent(IN)::VC
integer::i
real(kind(0.d0)),dimension(NIBase+1)::IBA,ICA
character(len=1024)::work
integer::position

  IBA=(/(i,i=0,NIBase)/)*IBDelta

  do i=0,NIBase
    ICA(i+1)=BJT_IC_vs_VC(VC,IBA(i+1))
!    print *,VC,IBA(i+1),ICA(i+1);pause!QQQQ
  end do

!Write output line to text buffer
  write(work,1000) VC,ICA*1000.d0	!Output in mA

!Strip output line of blanks, one by one
  work=trim(adjustl(work))	!Start by eliminating leading blanks
  do					!Strip out blanks, one by one
    position=index(trim(work)," ")	!Find blank, if any
    if(position == 0) exit		!If none, we are done
    work=work(1:position-1)//work(position+1:len_trim(work))
  end do

  write(lunout,2000) trim(work)

1000 format(f5.1,20(:",",f5.1))
2000 format(a)
end subroutine model_line
!----------------------------------------
end subroutine BJT_vi_Curves
!----------------------------------------
real(kind(0.d0)) function BJT_IC_vs_VC(VC,IB)	!Collector current vs. collector voltage
!This is the main computational engine
real(kind(0.d0)),intent(IN)::VC,IB
real(kind(0.d0))::ISat,ICC,ICFactor

  ISat=VC/Saturation_resistance
  ICC=IC0+beta*IB
  ICFactor=1.d0+VC/V_CE_Early		!Early voltage factor  

  if(ISat < ICC) then
    BJT_IC_vs_VC=ISat*ICFactor
  else
    BJT_IC_vs_VC=ICC*ICFactor
  end if

end function BJT_IC_vs_VC
!----------------------------------------
real(kind(0.d0)) function BJT_Output_Admittance(VC,IB)
real(kind(0.d0)),intent(IN)::VC,IB
real(kind(0.d0))::ISat,ICC,ICFactor

  ISat=VC/Saturation_resistance
  ICC=IC0+beta*IB

  if(ISat < ICC) then
    ICFactor=1.d0+2.d0*VC/V_CE_Early		!Early voltage factor  
    BJT_Output_Admittance=ICFactor/Saturation_Resistance
  else
    ICFactor=1.d0+VC/V_CE_Early			!Early voltage factor  
    BJT_Output_Admittance=ICC/V_CE_Early	!Eq, (5.53) pl. 274
  end if

end function BJT_Output_Admittance
!----------------------------------------
end module BJT
